
; Based on most code found on:
; http://1amstudios.com/2014/12/07/c64-smooth-scrolling/
; https://www.lemon64.com/forum/viewtopic.php?t=35038&sid=a5620cbc27fb8101b089cac7d2a13274

; Rest of code programmed by Stone Oakvalley Studios April 2019
; Its purpose is to load a "fastloader" for 1541 drive, then load the "fb" from SD2IEC File Browser v1.0
; in a one-shot experience. Its kind of boring to type so much on C64, and basically I want to just
; load 1 program which sets up fastloader and the filebrowser for me automatically.

; Combined with the "autoboot.prg" with options "M" and "49152", the final autostarted
; fastloader capable, SD2IEC FB Filebrowser is a sheer breeze to load and get quickly going!

; Due to the overlapped nature of both fastloader and SD2IEC browser, we need to install
; a small separate loader hidden into tape buffer. See the "fb_loader.asm" for details.

*=$C000 ; sys49152

        
; Relocate Code for FastLoad. 
startaddress=faststart   ; = 49152+offset from where incbin included our binary in assembled prg file
storeaddress=$0838      ; = SYS2104
storeaddress2=$0C01     ; The remainder of bytes to poke in for fastloader 
storeaddress3=$0334     ; The loader for filebrowser
loopcount=4   ; Each count is 255 bytes max. So 4*255=1020, the remainder of 186 bytes are copied in a simpler way
              ; This routine only works if bytes to copy are MORE than 255! loopcount=0 wont do it!

movemem
        ldx #$00                  
        ldy #loopcount          

loop
        lda startaddress,x  
        sta storeaddress,x   
        inx
        bne loop

        inc loop+2        ; change highbyte of startaddress
        inc loop+5        ; change highbyte of storeaddress
        dey
        bne loop

        lda #>startaddress
        sta loop+2
        lda #>storeaddress
        sta loop+5
        lda #$00
        sta movemem+1
        lda #loopcount
        sta movemem+3 

; Then copy the remainder too that wasn't included in the x*255 loop above!
        ldx #00        
loop1
        txa      
        lda faststart+969,x        ; peek byte from pos and load into accumlator 1 by 1
        sta storeaddress2,x        ; poke byte to buffer 1 by 1, continue where we left in the first "loop" above
        inx             
        cpx #186                   ; Did we reach its remainder of bytes? - if not loop until we do! 
        bne loop1



;Next, relocate my SYS828 loader as well into tape buffer, but since this is below 255 bytes, we can do it more simpler
        ldx #00        
loop2
        txa                    
        lda loadstart,x            ; peek byte from pos and load into accumlator 1 by 1
        sta storeaddress3,x        ; poke byte to buffer 1 by 1 
        inx             
        cpx #34                    ; Did we reach its remainder of bytes? - if not loop until we do!  
        bne loop2

        JSR storeaddress           ; 1st call up the SYS2104 (that starts the Fastloader)
        JMP storeaddress3          ; 2nd call up the loader hidden in tape buffer that loads the SD2IEC browser 
        RTS

faststart
incbin "fastload_sys2104.prg",2 ; Starts with SYS2104
loadstart
incbin "fb_loader.prg",2    ;Original code loads to 820 (then starts with SYS820 which then boots the "b" (SD2IEC FileBrowser))


